import { Body, Controller, HttpCode, HttpStatus, Post, Req, UseGuards } from '@nestjs/common';
import { ApiBearerAuth, ApiTags } from '@nestjs/swagger';
import { Request } from 'express';
import { AuthenticatedUser, Public } from '~/common/decorators';
import { AccessTokenGuard } from '~/common/guards';
import { ApiDataResponse, ApiLangRequestHeader } from '~/core/decorators';
import { ResponseFactory } from '~/core/utils';
import {
  AppleLoginRequestDto,
  CreateUnverifiedUserRequestDto,
  DisableBiometricRequestDto,
  EnableBiometricRequestDto,
  ForgetPasswordRequestDto,
  GoogleLoginRequestDto,
  RefreshTokenRequestDto,
  SendForgetPasswordOtpRequestDto,
  SendLoginOtpRequestDto,
  SetEmailRequestDto,
  setJuniorPasswordRequestDto,
  SetPasscodeRequestDto,
  VerifyLoginOtpRequestDto,
  VerifyUserRequestDto,
} from '../dtos/request';
import { SendForgetPasswordOtpResponseDto, SendRegisterOtpResponseDto } from '../dtos/response';
import { LoginResponseDto } from '../dtos/response/login.response.dto';
import { IJwtPayload } from '../interfaces';
import { AuthService } from '../services';

@Controller('auth')
@ApiTags('Auth')
@ApiBearerAuth()
@ApiLangRequestHeader()
export class AuthController {
  constructor(private readonly authService: AuthService) {}
  @Post('register/otp')
  async register(@Body() createUnverifiedUserDto: CreateUnverifiedUserRequestDto) {
    const phoneNumber = await this.authService.sendRegisterOtp(createUnverifiedUserDto);
    return ResponseFactory.data(new SendRegisterOtpResponseDto(phoneNumber));
  }

  @Post('register/verify')
  async verifyUser(@Body() verifyUserDto: VerifyUserRequestDto) {
    const [res, user] = await this.authService.verifyUser(verifyUserDto);
    return ResponseFactory.data(new LoginResponseDto(res, user));
  }

  @Post('login/otp')
  @HttpCode(HttpStatus.NO_CONTENT)
  async sendLoginOtp(@Body() data: SendLoginOtpRequestDto) {
    return this.authService.sendLoginOtp(data);
  }

  @Post('login/verify')
  @HttpCode(HttpStatus.OK)
  @ApiDataResponse(LoginResponseDto)
  async verifyLoginOtp(@Body() data: VerifyLoginOtpRequestDto) {
    const [token, user] = await this.authService.verifyLoginOtp(data);
    return ResponseFactory.data(new LoginResponseDto(token, user));
  }

  @Post('login/google')
  @HttpCode(HttpStatus.OK)
  @ApiDataResponse(LoginResponseDto)
  async loginWithGoogle(@Body() data: GoogleLoginRequestDto) {
    const [token, user] = await this.authService.loginWithGoogle(data);
    return ResponseFactory.data(new LoginResponseDto(token, user));
  }

  @Post('login/apple')
  @HttpCode(HttpStatus.OK)
  @ApiDataResponse(LoginResponseDto)
  async loginWithApple(@Body() data: AppleLoginRequestDto) {
    const [token, user] = await this.authService.loginWithApple(data);
    return ResponseFactory.data(new LoginResponseDto(token, user));
  }

  @Post('register/set-email')
  @HttpCode(HttpStatus.NO_CONTENT)
  @UseGuards(AccessTokenGuard)
  async setEmail(@AuthenticatedUser() { sub }: IJwtPayload, @Body() setEmailDto: SetEmailRequestDto) {
    await this.authService.setEmail(sub, setEmailDto);
  }

  @Post('register/set-passcode')
  @HttpCode(HttpStatus.NO_CONTENT)
  @UseGuards(AccessTokenGuard)
  async setPasscode(@AuthenticatedUser() { sub }: IJwtPayload, @Body() { passcode }: SetPasscodeRequestDto) {
    await this.authService.setPasscode(sub, passcode);
  }

  // @Post('register/set-phone/otp')
  // @UseGuards(AccessTokenGuard)
  // async setPhoneNumber(
  //   @AuthenticatedUser() { sub }: IJwtPayload,
  //   @Body() setPhoneNumberDto: CreateUnverifiedUserRequestDto,
  // ) {
  //   const phoneNumber = await this.authService.setPhoneNumber(sub, setPhoneNumberDto);
  //   return ResponseFactory.data(new SendRegisterOtpResponseDto(phoneNumber));
  // }

  // @Post('register/set-phone/verify')
  // @HttpCode(HttpStatus.NO_CONTENT)
  // @UseGuards(AccessTokenGuard)
  // async verifyPhoneNumber(@AuthenticatedUser() { sub }: IJwtPayload, @Body() { otp }: VerifyOtpRequestDto) {
  //   await this.authService.verifyPhoneNumber(sub, otp);
  // }

  @Post('biometric/enable')
  @HttpCode(HttpStatus.NO_CONTENT)
  @UseGuards(AccessTokenGuard)
  enableBiometric(@AuthenticatedUser() { sub }: IJwtPayload, @Body() enableBiometricDto: EnableBiometricRequestDto) {
    return this.authService.enableBiometric(sub, enableBiometricDto);
  }

  @Post('biometric/disable')
  @HttpCode(HttpStatus.NO_CONTENT)
  @UseGuards(AccessTokenGuard)
  disableBiometric(@AuthenticatedUser() { sub }: IJwtPayload, @Body() disableBiometricDto: DisableBiometricRequestDto) {
    return this.authService.disableBiometric(sub, disableBiometricDto);
  }

  @Post('forget-password/otp')
  async forgetPassword(@Body() sendForgetPasswordOtpDto: SendForgetPasswordOtpRequestDto) {
    const email = await this.authService.sendForgetPasswordOtp(sendForgetPasswordOtpDto);
    return ResponseFactory.data(new SendForgetPasswordOtpResponseDto(email));
  }

  @Post('forget-password/reset')
  @HttpCode(HttpStatus.NO_CONTENT)
  resetPassword(@Body() forgetPasswordDto: ForgetPasswordRequestDto) {
    return this.authService.verifyForgetPasswordOtp(forgetPasswordDto);
  }

  @Post('junior/set-passcode')
  @HttpCode(HttpStatus.NO_CONTENT)
  @Public()
  setJuniorPasscode(@Body() setPasscodeDto: setJuniorPasswordRequestDto) {
    return this.authService.setJuniorPasscode(setPasscodeDto);
  }

  @Post('refresh-token')
  @Public()
  async refreshToken(@Body() { refreshToken }: RefreshTokenRequestDto) {
    const [res, user] = await this.authService.refreshToken(refreshToken);
    return ResponseFactory.data(new LoginResponseDto(res, user));
  }

  @Post('logout')
  @HttpCode(HttpStatus.NO_CONTENT)
  @UseGuards(AccessTokenGuard)
  async logout(@Req() request: Request) {
    await this.authService.logout(request);
  }
}
