import { ApiProperty, PickType } from '@nestjs/swagger';
import {
  IsDateString,
  IsEnum,
  IsNotEmpty,
  IsNumberString,
  IsOptional,
  IsString,
  MaxLength,
  MinLength,
} from 'class-validator';
import { i18nValidationMessage as i18n } from 'nestjs-i18n';
import { CountryIso } from '~/common/enums';
import { DEFAULT_OTP_LENGTH } from '~/common/modules/otp/constants';
import { IsAbove18 } from '~/core/decorators/validations';
import { CreateUnverifiedUserRequestDto } from './create-unverified-user.request.dto';

export class VerifyUserRequestDto extends PickType(CreateUnverifiedUserRequestDto, ['email']) {
  @ApiProperty({ example: 'John' })
  @IsString({ message: i18n('validation.IsString', { path: 'general', property: 'customer.firstName' }) })
  @IsNotEmpty({ message: i18n('validation.IsNotEmpty', { path: 'general', property: 'customer.firstName' }) })
  firstName!: string;

  @ApiProperty({ example: 'Doe' })
  @IsString({ message: i18n('validation.IsString', { path: 'general', property: 'customer.lastName' }) })
  @IsNotEmpty({ message: i18n('validation.IsNotEmpty', { path: 'general', property: 'customer.lastName' }) })
  lastName!: string;

  @ApiProperty({ example: '2021-01-01' })
  @IsDateString({}, { message: i18n('validation.IsDateString', { path: 'general', property: 'customer.dateOfBirth' }) })
  @IsAbove18({ message: i18n('validation.IsAbove18', { path: 'general', property: 'customer.dateOfBirth' }) })
  dateOfBirth!: Date;

  @ApiProperty({ example: 'JO' })
  @IsEnum(CountryIso, {
    message: i18n('validation.IsEnum', { path: 'general', property: 'customer.countryOfResidence' }),
  })
  @IsOptional()
  countryOfResidence: CountryIso = CountryIso.SAUDI_ARABIA;

  @ApiProperty({ example: '111111' })
  @IsNumberString(
    { no_symbols: true },
    { message: i18n('validation.IsNumberString', { path: 'general', property: 'auth.otp' }) },
  )
  @MaxLength(DEFAULT_OTP_LENGTH, {
    message: i18n('validation.MaxLength', { path: 'general', property: 'auth.otp', length: DEFAULT_OTP_LENGTH }),
  })
  @MinLength(DEFAULT_OTP_LENGTH, {
    message: i18n('validation.MinLength', { path: 'general', property: 'auth.otp', length: DEFAULT_OTP_LENGTH }),
  })
  otp!: string;
}
