// notification-created.handler.ts
import { MailerService } from '@nestjs-modules/mailer';
import { Injectable, Logger } from '@nestjs/common';
import { SendEmailRequestDto } from '~/common/modules/notification/dtos/request';
import { EventType, NotificationChannel, NotificationScope } from '~/common/modules/notification/enums';
import { FirebaseService, TwilioService } from '~/common/modules/notification/services';
import { IEventInterface } from '~/common/redis/interface';
import { DeviceService } from '~/user/services';

@Injectable()
export class NotificationCreatedListener {
  private readonly logger = new Logger(NotificationCreatedListener.name);

  constructor(
    private readonly twilioService: TwilioService,
    private readonly deviceService: DeviceService,
    private readonly mailerService: MailerService,
    private readonly firebaseService: FirebaseService,
  ) {}

  /**
   * Handles the NOTIFICATION_CREATED event by calling the appropriate channel logic.
   */
  async handle(event: IEventInterface) {
    this.logger.log(
      `Handling ${EventType.NOTIFICATION_CREATED} event for notification ${event.id} (channel: ${event.channel})`,
    );

    switch (event.channel) {
      case NotificationChannel.SMS:
        return this.sendSMS(event.recipient!, event.message);

      case NotificationChannel.PUSH:
        return this.sendPushNotification(event.userId, event.title, event.message);

      case NotificationChannel.EMAIL:
        return this.sendEmail({
          to: event.recipient!,
          subject: event.title,
          template: this.getTemplateFromNotification(event),
          data: event.data,
        });
    }
  }

  private getTemplateFromNotification(notification: IEventInterface) {
    switch (notification.scope) {
      case NotificationScope.OTP:
        return 'otp';
      case NotificationScope.USER_INVITED:
        return 'user-invite';
      default:
        return 'otp';
    }
  }

  private async sendPushNotification(userId: string, title: string, body: string) {
    this.logger.log(`Sending push notification to user ${userId}`);
    const tokens = await this.deviceService.getTokens(userId);

    if (!tokens.length) {
      this.logger.log(`No device tokens found for user ${userId}, but notification was created in the DB.`);
      return;
    }
    return this.firebaseService.sendNotification(tokens, title, body);
  }

  private async sendSMS(to: string, body: string) {
    this.logger.log(`Sending SMS to ${to}`);
    await this.twilioService.sendSMS(to, body);
  }

  private async sendEmail({ to, subject, data, template }: SendEmailRequestDto) {
    this.logger.log(`Sending email to ${to}`);
    try {
      await this.mailerService.sendMail({
        to,
        subject,
        template,
        context: { ...data, currentYear: new Date().getFullYear() },
      });
      this.logger.log(`Email sent to ${to}`);
    } catch (error) {
      this.logger.error(`Failed to send email to ${to} error: ${JSON.stringify(error)}`);
      throw error;
    }
  }
}
